/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  STACK.C -- processing functions for an integer  version 1.0  *
 *             stack                                             *
 *  Operations:                                                  *
 *    Provides basic stack initialization and push and pop for   *
 *    stacks of integers.                                        *
 *                                                               *
 *  Language:     Borland C                                      *
 *  System:       Gateway 386/25, MS-DOS 5.0                     *
 *  Application:  CSE 784/791                                    *
 *  Author:       Jim Fawcett                                    *
 *                                                               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <stdio.h>     /* ANSI:     fputs  */
#include <stdlib.h>    /* ANSI:     exit   */
#include "stack.h"

/*----< void init_stack(int stack[]) >---------------------------*/

/*  Processing:                                                  */
/*  - initialize stack pointer                                   */
/*  Inputs:                                                      */
/*  - stack, an array of ints                                    */
/*  Outputs:                                                     */
/*  - initialized stack pointer = stack[0]                       */

void init_stack(int stack[]) {

  stack[0] = 1;
}


/*----< void push(int data, int stack[], int size) >-------------*/

/*  Processing:                                                  */
/*  - add an integer data to the top of the stack                */
/*  - check for stack overflow                                   */
/*  Inputs:                                                      */
/*  - data, an integer value to be placed on the stack           */
/*  - stack, an array of integers                                */
/*  - size, the size of array                                    */
/*  Outputs:                                                     */
/*  - incremented stack pointer, stack[0]                        */

void push(int data, int stack[], int size) {

  if ( stack[0] > size) {
    fputs("stack overflow\n",stderr);
    exit(1);
  }
  stack[stack[0]++] = data;
}


/*----< int pop(int stack[]) >-----------------------------------*/

/*  Processing:                                                  */
/*  - remove an integer, data, from the top of the stack         */
/*  - check for stack underflow                                  */
/*  Inputs:                                                      */
/*  - stack, an array of integers                                */
/*  Outputs:                                                     */
/*  - an integer data value is returned, and stack[0] is decr    */

int pop(int stack[]) {

  if (stack[0] < 2) {
    fputs("stack underflow\n", stderr);
    exit(1);
  }
  return stack[--stack[0]];
}

/*----< void show_stack(int stack[]) >-----------------------*/

/*  Processing:                                              */
/*  - display contents of stack                              */
/*  Inputs:                                                  */
/*  - stack, an array of integers                            */
/*  Outputs:                                                 */
/*  - display stack contents on stdout                       */

void show_stack(int stack[]) {

  #define STEP 10
  int i;

  fprintf(stdout,"\ntop of stack on right:\n");
  for (i=0; i<stack[0]-1; i++)
    fprintf(stdout,"  %d",stack[i+1]);
  #undef STEP
}

/**/
#ifdef test_stack

/*----< test stub >------------------------------------------*/

void main() {

  #define STRING_SIZE 20
  #define STACK_SIZE  20
  char input[STRING_SIZE], ch;
  int stack[STACK_SIZE], data;

  init_stack(stack);
  while(1) {  /* do forerver */
    fputs("\nenter stack element: ", stdout);
    fgets(input,STRING_SIZE,stdin);
    data = atoi(input);
    push(data, stack, STACK_SIZE);
    show_stack(stack);
    fputs("\ncontinue (y,n) ? ", stdout);
    ch = getc(stdin);
    if ((ch == 'N') || (ch == 'n'))
      break;
  }
  while(1) {
    show_stack(stack);
    fputs("\npoping stack: ", stdout);
    pop(stack);
  }
}
#endif